# Generated by Django 4.2.15 on 2024-11-10 19:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("NEMO_floor_plans", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="floorplannode",
            name="sensor_text_position",
            field=models.IntegerField(
                blank=True,
                choices=[(0, "Top"), (1, "Right"), (2, "Bottom"), (3, "Left")],
                help_text="The position of the sensor text",
                null=True,
            ),
        ),
    ]
